import java.lang.reflect.Method;
import java.util.Random;

import decisionanalysis.DecisionTree;


final class Test extends Object
{
    private static Random rnd;

    public static void main(String[] args)
    {
	int numTrials = 100;
	long seed = System.currentTimeMillis();
	boolean verbose = false;
	if (args.length < 1)
	{
	    System.err.println("args: <model> [<# trials> <seed> v]");
	    return;
	}
	String modelPath = "../models/" + args[0] + ".dt";
	if (args.length > 1)
	    numTrials = Integer.parseInt(args[1]);
	if (args.length > 2)
	    seed = Integer.parseInt(args[2]);
	if (args.length > 3)
	    verbose = (args[3].charAt(0) == 'v');
	System.out.println("> " + args[0] + ' ' + numTrials + ' ' + seed);

	rnd = new Random(seed);
	DTPronouncer pronouncer = new DTPronouncer();
	int modelId = pronouncer.loadModel(modelPath);
	if (modelId < 0)
	{
	    System.err.println("could not load model \"" + modelPath + "\"!");
	    return;
	}

	long sum = 0;
	long sqsum = 0;
	Class[] params = { DTPronouncer.class, Integer.TYPE };
	Method makeModel = null;
	try
	{
	    makeModel = Test.class.getMethod(args[0], params);
	}
	catch (NoSuchMethodException e)
	{
	    System.err.println("model \"" + args[0] + "\" not supported!");
	    return;
	}
	Object[] methodArgs = { pronouncer, new Integer(modelId) };
	for (int i = -1; i < numTrials; i++)
	{
	    long start = System.currentTimeMillis();
	    try
	    {
		makeModel.invoke(null, methodArgs);
	    }
	    catch (Exception e)
	    {
		System.err.println(e);
		return;
	    }
	    int advice = pronouncer.evaluateModel(modelId);
	    long stop = System.currentTimeMillis();
	    long m = stop - start;
	    if (i >= 0)
	    {
		if (verbose)
		    System.out.println(m + " (" + advice + ')');
		sum += m;
		sqsum += m * m;
	    }
	}
	System.out.println("mean is " + ((double)sum / numTrials) + " ms");
	System.out.println("deviation is " +
			   (Math.sqrt(sqsum - sum * sum / (double)numTrials) /
			    numTrials));
    }

    private static void getProbabilities(int p, int s, double[] probs, int f)
    {
	while (p-- > 0)
	{
	    int i = s - 1;
	    double last = 1.0;
	    while (i-- > 0)
		last -= (probs[f++] = last * rnd.nextDouble());
	    probs[f++] = last;
	}
    }

    private static void getUtilities(int p, double[] utils, int f)
    {
	while (p-- > 0)
	    utils[f++] = rnd.nextDouble();
    }

    public static void cantsee1(DTPronouncer pronouncer, int modelId)
    {
	double[] values = new double[8];
	getProbabilities(2, 2, values, 0);
	getUtilities(4, values, 4);
	pronouncer.setModelValues(modelId, values, 0);
    }

    public static void cantsee2(DTPronouncer pronouncer, int modelId)
    {
	double[] values = new double[14];
	getProbabilities(1, 2, values, 0);
	getProbabilities(4, 2, values, 2);
	getUtilities(4, values, 10);
	pronouncer.setModelValues(modelId, values, 0);
    }

    public static void hasball1(DTPronouncer pronouncer, int modelId)
    {
	double[] values = new double[24];
	getProbabilities(3, 4, values, 0);
	getUtilities(12, values, 12);
	pronouncer.setModelValues(modelId, values, 0);
    }

    public static void hasball2(DTPronouncer pronouncer, int modelId)
    {
	double[] values = new double[64];
	getProbabilities(1, 2, values, 0);
	getProbabilities(1, 2, values, 2);
	getProbabilities(12, 4, values, 4);
	getUtilities(12, values, 52);
	pronouncer.setModelValues(modelId, values, 0);
    }
}
